document.addEventListener('DOMContentLoaded', function () {
  var currentPage = Number(document.getElementById('currentPage').textContent)
  var paginationLimit = Number(document.getElementById('limitSelect').value)

  setTimeout(function () {
    fetchUsers(currentPage, paginationLimit)
  }, 2000)

  document.getElementById('limitSelect').addEventListener('change', function () {
    var paginationLimit = Number(document.getElementById('limitSelect').value)
    document.getElementById('currentPage').innerText = 1
    fetchUsers(1, paginationLimit)
  })

  document.getElementById('prevPage').addEventListener('click', function () {
    var paginationLimit = Number(document.getElementById('limitSelect').value)
    var currentPage = Number(document.getElementById('currentPage').textContent)
    if (currentPage > 1) {
      currentPage--
      fetchUsers(currentPage, paginationLimit)
    }
  })

  document.getElementById('nextPage').addEventListener('click', function () {
    var paginationLimit = Number(document.getElementById('limitSelect').value)
    var currentPage = Number(document.getElementById('currentPage').textContent)
    var totalPageStr = document.getElementById('totalPages').textContent
    totalPageStr = totalPageStr.replace('/', '')
    currentPage++
    var totalPage = Number(totalPageStr)
    if (currentPage > totalPage) {
      return
    }
    fetchUsers(currentPage, paginationLimit)
  })

  document.getElementById('filterForm').addEventListener('submit', function (event) {
    event.preventDefault()

    var fullName = document.getElementById('fullNameInput').value
    var company = document.getElementById('companyInput').value
    var email = document.getElementById('emailInput').value
    document.getElementById('limitSelectDiv').style.display = 'none'
    document.getElementById('enableDisableSelect').value = 'ENABLED'

    fetchFilteredUsers(fullName, company, email, currentPage)
    document.getElementById('fullNameInput').value = ''
    document.getElementById('companyInput').value = ''
    document.getElementById('emailInput').value = ''
  })

  document.getElementById('resetUserTableBtn').addEventListener('click', function () {
    // event.preventDefault()
    document.getElementById('currentPage').innerText = 1
    document.getElementById('fullNameInput').value = ''
    document.getElementById('companyInput').value = ''
    document.getElementById('emailInput').value = ''
    document.getElementById('limitSelectDiv').style.display = 'flex'
    document.getElementById('enableDisableSelect').value = 'ENABLED'
    fetchUsers(1, 20)
  })

  document.getElementById('enableDisableSelect').addEventListener('change', function (event) {
    var paginationLimit = Number(document.getElementById('limitSelect').value)
    var selectedOption = event.target.value
    selectDisableRender(selectedOption, paginationLimit)
  })
})

function selectDisableRender(selectedOption, limit) {
  if (selectedOption === 'DISABLED') {
    showUpdateLoading()
    document.getElementById('limitSelectDiv').style.display = 'none'
    fetch(
      'http://10.1.20.77:8080/portal/rest/v1/social/users?q=&isDisabled=true&status=DISABLED&userType=&offset=0&limit=20&returnSize=true',
      {
        method: 'GET'
      }
    )
      .then(function (response) {
        return response.json()
      })
      .then(function (data) {
        var allUsers = data.users
        var noOfUsers = allUsers.length
        var totalSize = data.size
        var noOfPages = Math.ceil(totalSize / limit)
        var noOfPages = Math.ceil(totalSize / totalSize)
        renderPaginationInfo(noOfUsers, noOfPages, totalSize, totalSize, 1, true)
        populateTable(allUsers)
      })
      .catch(function (error) {
        console.error('Error:', error)
      })
      .finally(function () {
        hideUpdateLoading()
      })
  } else if (selectedOption === 'ENABLED') {
    document.getElementById('limitSelectDiv').style.display = 'flex'
    fetchUsers(1, limit)
  }
}

function fetchFilteredUsers(fullName, company, email) {
  showLoading()
  var payload = {}
  if (fullName.trim() !== '') {
    payload['fullName'] = fullName
  }
  if (company.trim() !== '') {
    payload['company'] = company
  }
  if (email.trim() !== '') {
    payload['email'] = email
  }

  fetch(
    'http://10.1.20.77:8080/portal/rest/v1/social/users/advancedfilter?offset=0&limit=1000&expand=all,spacesCount,relationshipStatus,connectionsCount,binding&filterType=all&returnSize=true',
    {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json'
      },
      body: JSON.stringify(payload)
    }
  )
    .then(function (response) {
      return response.json()
    })
    .then(function (data) {
      var allUsers = data.users
      document.getElementById('currentPage').innerText = 1
      var noOfUsers = allUsers.length
      var totalSize = data.size
      var noOfPages = Math.ceil(totalSize / totalSize)
      renderPaginationInfo(noOfUsers, noOfPages, totalSize, totalSize, 1, true)
      populateTable(allUsers)
    })
    .catch(function (error) {
      console.error('Error:', error)
    })
    .finally(function () {
      hideLoading()
    })
}

function fetchUsers(currentPage, limit) {
  showLoading()
  fetch('http://10.1.20.77:8080/portal/rest/v1/social/users?limit=1000&returnSize=true&excludeCurrentUser=true', {
    method: 'GET'
  })
    .then(function (response) {
      return response.json()
    })
    .then(function (data) {
      var allUsers = data.users
      var noOfUsers = allUsers.length
      var totalSize = data.size
      var noOfPages = Math.ceil(totalSize / limit)
      renderPaginationInfo(noOfUsers, noOfPages, totalSize, limit, currentPage, false)
      renderUsersForPage(allUsers, currentPage, limit)
    })
    .catch(function (error) {
      console.error('Error:', error)
    })
    .finally(function () {
      hideLoading()
    })
}

function renderUsersForPage(allUsers, page = 1, limit = 500) {
  var start = (page - 1) * limit
  var end = start + limit
  var usersToDisplay = allUsers.slice(start, end)
  populateTable(usersToDisplay)

  document.getElementById('currentPage').innerText = page
}

function populateTable(users) {
  var enabledIcon = 'Enabled'
  var disabledIcon = 'Disabled'
  var table = document.getElementById('userTable')
  table.innerHTML =
    '<tr><th class="selectCol"><label class="custom-checkbox"><input type="checkbox" name="selectRow" id="selectRow" /><span class="checkmark"></span></label></th><th>Company</th><th>First Name</th><th>Last Name</th><th>Email</th><th>Type</th><th>Last Login Time</th><th>Enabled</th></tr>'

  users.forEach(function (user, index) {
    var row = table.insertRow(-1)
    var checkboxCell = row.insertCell(0)
    var companyCell = row.insertCell(1)
    var firstNameCell = row.insertCell(2)
    var lastNameCell = row.insertCell(3)
    var emailCell = row.insertCell(4)
    var isInternalCell = row.insertCell(5)
    var lastLoginTimeCell = row.insertCell(6)
    var enabledCell = row.insertCell(7)

    checkboxCell.innerHTML = `<label class="custom-checkbox"><input type="checkbox" name="selectRow" class="selectEachRow" data-email="${user.email}" data-id="${index}" /><span class="checkmark"></span></label>`
    firstNameCell.innerHTML = user.firstname || ''
    lastNameCell.innerHTML = user.lastname || ''
    companyCell.innerHTML = user.company || ''
    emailCell.innerHTML = user.email || ''
    isInternalCell.innerHTML = user.company === 'DRB-HICOM Berhad' ? 'Internal' : 'External'
    lastLoginTimeCell.innerHTML = user.lastLoginTime ? formatDateTime(user.lastLoginTime) : ''
    enabledCell.innerHTML = user.enabled ? enabledIcon : disabledIcon
  })

  var selectAllCheckbox = document.getElementById('selectRow')
  if (selectAllCheckbox) {
    selectAllCheckbox.removeEventListener('click', handleSelectAllClick)
    selectAllCheckbox.addEventListener('click', handleSelectAllClick)
  }

  document.getElementById('enabledBtn').addEventListener('click', function (event) {
    const selectedEmails = []
    const selectedIndexes = []
    document.querySelectorAll('.selectEachRow:checked').forEach(function (checkbox) {
      selectedEmails.push(checkbox.dataset.email)
      selectedIndexes.push(checkbox.dataset.id)
    })
    const rows = document.querySelectorAll('tbody > tr')
    const rowsArray = Array.from(rows)
    rowsArray.shift()
    selectedIndexes.forEach(function (id) {
      const row = rowsArray[id]
      row.cells[7].innerHTML = enabledIcon
    })
    showUpdateLoading()
    fetch('http://10.1.20.77:8080/portal/rest/v1/social/users/bulk/enable', {
      method: 'PATCH',
      credentials: 'include',
      headers: {
        'Content-Type': 'application/json'
      },
      body: JSON.stringify(selectedEmails)
    })
      .then(function (response) {
        response.json()
      })
      .catch(function (error) {
        console.error('Error:', error)
      })
      .finally(function () {
        hideUpdateLoading()
      })
  })

  document.getElementById('disabledBtn').addEventListener('click', function (event) {
    const selectedEmails = []
    const selectedIndexes = []
    document.querySelectorAll('.selectEachRow:checked').forEach(function (checkbox) {
      selectedEmails.push(checkbox.dataset.email)
      selectedIndexes.push(checkbox.dataset.id)
    })
    const rows = document.querySelectorAll('tbody > tr')
    const rowsArray = Array.from(rows)
    rowsArray.shift()
    selectedIndexes.forEach(function (id) {
      const row = rowsArray[id]
      row.cells[7].innerHTML = disabledIcon
    })
    showUpdateLoading()
    fetch('http://10.1.20.77:8080/portal/rest/v1/social/users/bulk/disable', {
      method: 'PATCH',
      credentials: 'include',
      headers: {
        'Content-Type': 'application/json'
      },
      body: JSON.stringify(selectedEmails)
    })
      .then(function (response) {
        response.json()
      })
      .catch(function (error) {
        console.error('Error:', error)
      })
      .finally(function () {
        hideUpdateLoading()
      })
  })
}

function formatDateTime(timestamp) {
  var date = new Date(parseInt(timestamp, 10))
  var months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
  var year = date.getFullYear()
  var month = months[date.getMonth()]
  var day = date.getDate()
  var hour = date.getHours()
  var minute = date.getMinutes()
  var ampm = hour >= 12 ? 'PM' : 'AM'
  hour = hour % 12
  hour = hour ? hour : 12
  minute = minute < 10 ? '0' + minute : minute

  var formattedTime = month + ' ' + day + ', ' + year + ', ' + hour + ':' + minute + ' ' + ampm
  return formattedTime
}

function renderPaginationInfo(noOfUsers, noOfPages, totalSize, limit, currentPage, filterEnable = false) {
  var usersFrom = (currentPage - 1) * limit + 1
  var usersTo = usersFrom + limit - 1
  if (usersTo > totalSize) {
    usersTo = totalSize
  }
  if (!noOfPages) {
    noOfPages = 1
  }
  if (noOfUsers === 0) {
    document.getElementById('zeroUsers').style.display = 'block'
    document.getElementById('paginationInfo').style.display = 'none'
    document.getElementById('showingFilterUsers').style.display = 'none'
  } else {
    document.getElementById('zeroUsers').style.display = 'none'
    document.getElementById('paginationInfo').style.display = 'block'
    document.getElementById('showingFilterUsers').style.display = 'none'
    if (filterEnable) {
      document.getElementById('showingFilterUsers').style.display = 'block'
    }
  }
  document.getElementById('usersRange').innerText = usersFrom + ' - ' + usersTo
  document.getElementById('totalPages').innerText = noOfPages
  document.getElementById('totalUsers').innerText = totalSize
}

function showLoading() {
  console.log('first show loading')
  document.getElementById('loadingDiv').style.display = 'block'
  document.getElementById('tableContainer').style.display = 'none'
}

function hideLoading() {
  console.log('first hide loading')
  document.getElementById('loadingDiv').style.display = 'none'
  document.getElementById('tableContainer').style.display = 'table'
}

function showUpdateLoading() {
  console.log('show loading update')
  document.getElementById('updateDiv').style.display = 'block'
  document.getElementById('tableContainer').style.display = 'none'
}

function hideUpdateLoading() {
  console.log('hide loading update')
  document.getElementById('updateDiv').style.display = 'none'
  document.getElementById('tableContainer').style.display = 'table'
}

function handleSelectAllClick(event) {
  var checkboxes = document.querySelectorAll('.selectEachRow')
  checkboxes.forEach(function (checkbox) {
    checkbox.checked = event.target.checked
  })
}
